-- FertErazor
-- Spec for removing fertilizer from unseeded areas after harvesting
-- V1.0
-- 29.11.13


FertErazor = {};

function FertErazor.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Combine, specializations);
end;

function FertErazor:load(xmlFile)
	
	self.getCuttingAreas = SpecializationUtil.callSpecializationsFunction("self.getCuttingAreas");
	self.wwMinMaxAI = SpecializationUtil.callSpecializationsFunction("self.wwMinMaxAI");
	self.wwMinMaxAreas = SpecializationUtil.callSpecializationsFunction("self.wwMinMaxAreas");
	self.createDestroyArea = SpecializationUtil.callSpecializationsFunction("self.createDestroyArea");
	self.setDestroyArea = SpecializationUtil.callSpecializationsFunction("self.setDestroyArea");
	
	self.getCuttingAreas = FertErazor.getCuttingAreas;
	self.wwMinMaxAI = FertErazor.wwMinMaxAI;
	self.wwMinMaxAreas = FertErazor.wwMinMaxAreas;
	self.createDestroyArea = FertErazor.createDestroyArea;
	self.setDestroyArea = FertErazor.setDestroyArea;
	
	-- Areas
	
	self.wwMin = 0;
	self.wwMax = 0;
	self.wwY = 0;
	self.wwZ = 0;
	self.destroyOffsetZ = -1.5;
	
	self.destroyArea = {}
	self.destroyArea = self:createDestroyArea();
	
	if table.getn(self.cuttingAreas) > 0 then
		self:getCuttingAreas();
		--self.destroyArea = self:createDestroyArea();
		self:setDestroyArea();
	end;

end;

function FertErazor:delete()
end;

function FertErazor:readStream(streamId, connection)

end;

function FertErazor:writeStream(streamId, connection)

end;

function FertErazor:mouseEvent(posX, posY, isDown, isUp, button)

end;

function FertErazor:keyEvent(unicode, sym, modifier, isDown)
end;

function FertErazor:attachImplement(implement)
	self:getCuttingAreas();
	self:setDestroyArea();
end;

function FertErazor:onLeave()

end

function FertErazor:update(dt)
	
end;

function FertErazor:updateTick(dt)
	if self.isServer then
		if self:getIsActive() then
			if self.isThreshing and self.movingDirection > 0 then
				for _,area in pairs(self.destroyArea) do
					local x,y,z = getWorldTranslation(area.start);
					local x1,y1,z1 = getWorldTranslation(area.width);
					local x2,y2,z2 = getWorldTranslation(area.height);
					z,z1,z2 = worldToLocal(self.rootNode,z,z1,z2);
					z,z1,z2 = localToWorld(self.rootNode,z,z1,z2);
				
					local startX,startZ, widthX,widthZ, heightX,heightZ = Utils.getXZWidthAndHeight(g_currentMission.terrainDetailId, x, z, x1, z1, x2, z2);
					setDensityParallelogram(g_currentMission.terrainDetailId, startX, startZ, widthX, widthZ, heightX, heightZ, g_currentMission.sprayChannel, 1, 0);
				end;
			end;
		end;
	end;
end;


function FertErazor:draw()

end;

function FertErazor:getCuttingAreas()
	self.wwMin = 0;
	self.wwMax = 0;
	self.wwY = 0;
	self.wwZ = 0;
	self.wwMin,self.wwMax,self.wwY,self.wwZ = self:wwMinMaxAreas(self,self.cuttingAreas);
	for nIndex,oImplement in pairs(self.attachedImplements) do --parse all implements
		if oImplement ~= nil and oImplement.object ~= nil then
			self.wwMin,self.wwMax,self.wwY,self.wwZ = self:wwMinMaxAreas(self,oImplement.object.cuttingAreas);
		end;
	end;
	
	local workWidth = math.abs(self.wwMax-self.wwMin);
	if workWidth > .1 then
		self.workWidth = workWidth;
		self.wwCenter = (self.wwMin+self.wwMax)/2;
		if math.abs(self.wwCenter) < 0.1 then
			self.wwCenter = 0;
		end;
	end;
	--print("Get cutting areas complete!");
	--print("Arbeitsbreite = "..tostring(self.workWidth));
end;

function FertErazor:wwMinMaxAreas(self,areas)

	if areas ~= nil then
		for _,cuttingArea in pairs(areas) do
			if self:getIsAreaActive(cuttingArea) then

				local x1,y1,z1 = getWorldTranslation(cuttingArea.start)
				local x2,y2,z2 = getWorldTranslation(cuttingArea.width)
				local x3,y3,z3 = getWorldTranslation(cuttingArea.height)
				local lx1,ly1,lz1 = worldToLocal(self.rootNode,x1,y1,z1)
				local lx2,ly2,lz2 = worldToLocal(self.rootNode,x2,y2,z2)
				local lx3,ly3,lz3 = worldToLocal(self.rootNode,x3,y3,z3)
				local caParentId = getParent(cuttingArea.start)
				
				if lx1 < self.wwMin then
					self.wwMin = lx1;
				end
				if lx1 > self.wwMax then
					self.wwMax = lx1;
				end
				if lx2 < self.wwMin then
					self.wwMin = lx2;
				end
				if lx2 > self.wwMax then
					self.wwMax = lx2;
				end
				if lx3 < self.wwMin then
					self.wwMin = lx3;
				end
				if lx3 > self.wwMax then
					self.wwMax = lx3;
				end
				self.wwY = ly1;
				self.wwZ = lz1;
				self.caParentId = caParentId;
			end
		end
	end
	--print("MinMaxArea complete!");
	return self.wwMin, self.wwMax, self.wwY, self.wwZ;

end;

function FertErazor:createDestroyArea()
	local destroyArea = {};
	local startId = createTransformGroup("start");
	link(self.components[1].node, startId);
	local heightId = createTransformGroup("height");
	link(self.components[1].node, heightId);
	local widthId = createTransformGroup("width");
	link(self.components[1].node, widthId);

	table.insert(destroyArea, {foldMinLimit=0,start=startId,height=heightId,foldMaxLimit=0.2,width=widthId});

	--print("Created Destroy Area!");
	return destroyArea;
end;

function FertErazor:setDestroyArea()
	local xMin = self.wwMin;
	local xMax = self.wwMax;
	local y = self.wwY;
	local z = self.wwZ + self.destroyOffsetZ;
	local hz = z - 2;

	setTranslation(self.destroyArea[1].start,xMax,y,z);
	setTranslation(self.destroyArea[1].height,xMax,y,hz);
	setTranslation(self.destroyArea[1].width,xMin,y,z);

	--print("Set Destroy Area!");
end;